package org.ow2.bonita.util.xcmis;

import java.io.IOException;
import java.util.Properties;

import org.ow2.bonita.services.DocumentationManager;
import org.ow2.bonita.util.EnvTool;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class Configuration {
  
  protected static final String JDBC_ORACLE_DRIVER = "oracle.jdbc.OracleDriver";
  protected static final String JDBC_POSTGRESQL_DRIVER = "org.postgresql.Driver";
  protected static final String JDBC_SQLSERVER_DRIVER = "net.sourceforge.jtds.jdbc.Driver";
  protected static final String JDBC_MYSSQL_DRIVER = "com.mysql.jdbc.Driver";
  protected static final String JDBC_HSSQL_DRIVER = "org.hsqldb.jdbcDriver";

  private static final Logger LOG = LoggerFactory.getLogger(Configuration.class);

  private static final String HSQL_PROPERTIES_FILE_NAME = "bos.xcmis.migration.hsql.properties";
  private static final String MYSQL_PROPERTIES_FILE_NAME = "bos.xcmis.migration.mysql.properties";
  private static final String SQLSERVER_PROPERTIES_FILE_NAME = "bos.xcmis.migration.sqlserver.properties";
  private static final String ORACLE_PROPERTIES_FILE_NAME = "bos.xcmis.migration.oracle.properties";
  private static final String POSTGRESQL_PROPERTIES_FILE_NAME = "bos.xcmis.migration.postgresql.properties";
  
  private final Properties properties;

  public Configuration(final String dbDriver) throws ConfigurationException, IOException {
    // Read properties file.
    String propertiesPath = "/migration/";
    if (JDBC_ORACLE_DRIVER.equals(dbDriver)) {
      propertiesPath += ORACLE_PROPERTIES_FILE_NAME;
    } else if (JDBC_POSTGRESQL_DRIVER.equals(dbDriver)) {
      propertiesPath += POSTGRESQL_PROPERTIES_FILE_NAME;
    } else if (JDBC_MYSSQL_DRIVER.equals(dbDriver)) {
      propertiesPath += MYSQL_PROPERTIES_FILE_NAME;
    } else if (JDBC_SQLSERVER_DRIVER.equals(dbDriver)) {
      propertiesPath += SQLSERVER_PROPERTIES_FILE_NAME;
    } else if (JDBC_HSSQL_DRIVER.equals(dbDriver)){
      propertiesPath += HSQL_PROPERTIES_FILE_NAME;
    } else {
      final StringBuilder stb = new StringBuilder();
      stb.append("Unable to find properties file associated to the driver class '");
      stb.append("'.  The list of allowed drivers is ");
      stb.append(JDBC_HSSQL_DRIVER);
      stb.append(", ");
      stb.append(JDBC_MYSSQL_DRIVER);
      stb.append(", ");
      stb.append(JDBC_ORACLE_DRIVER);
      stb.append(", ");
      stb.append(JDBC_POSTGRESQL_DRIVER);
      stb.append(", ");
      stb.append(JDBC_SQLSERVER_DRIVER);
      throw new ConfigurationException(stb.toString());
    }
    this.properties = new Properties();
    this.properties.load(this.getClass().getResourceAsStream(propertiesPath));
  }

  public final String getQuery(final String queryName) {
    return this.properties.getProperty("query." + queryName);
  }

  public long getPageSize() {
    return Long.valueOf(properties.getProperty("page.size"));
  }

  public DocumentationManager getDocumentManager() {
    return EnvTool.getDocumentationManager();
  }
}
